@@ -14,7 +14,7 @@ package MCE::Core::Input::Handle;
 use strict;
 use warnings;
 
-our $VERSION = '1.824';
+our $VERSION = '1.825';
 
 ## Items below are folded into MCE.
 
@@ -64,9 +64,8 @@ sub _worker_read_handle {
 
    if ($_lock_chn) {
       # inlined for performance
-      if ($self->{_data_channels} > 7) {
-         my $_chn   = $self->{_wid} % 7 + 1;
-         $_DAT_LOCK = $self->{'_mutex_'.$_chn};
+      if ($self->{_data_channels} > 6) {
+         $_DAT_LOCK = $self->{'_mutex_'.( $self->{_wid} % 6 + 1 )};
       }
       $_dat_ex = sub {
          1 until sysread($_DAT_LOCK->{_r_sock}, my($_b), 1) || ($! && !$!{'EINTR'});
